//
//  menuCell.m
//  eimEngine
//
//  Created by CLARKYJ on 17/1/14.
//  Copyright © 2017年 tencent. All rights reserved.
//

#import "menuCell.h"

#define DEFAULT_SELECTION_BG_COLOR (RGBACOLOR(0xde*255, 0xdf*255, 0xe0*255, 1))

#define SPACE 10

#define MENU_ICON_DEFAULT_WIDTH 24
#define ICON_FRAME (CGRectMake(SPACE, (MENU_CELL_DEFAULT_HEIGHT-MENU_ICON_DEFAULT_WIDTH)/2, MENU_ICON_DEFAULT_WIDTH, MENU_ICON_DEFAULT_WIDTH))

#define LABEl_WIDTH CELL_DEFAULT_WIDTH-MENU_ICON_DEFAULT_WIDTH-3*SPACE
#define LABEl_HEIGHT 20
#define LABEL_FRAME (CGRectMake(2*SPACE+MENU_ICON_DEFAULT_WIDTH, (MENU_CELL_DEFAULT_HEIGHT-LABEl_HEIGHT)/2, LABEl_WIDTH, LABEl_HEIGHT))
#define LABEL_COLOR (RGBACOLOR(0*255, 0*255, 0*255, 1))

@interface menuCell()
{
    UIView* _backgroundView;
}

@end

@implementation menuCell
@synthesize icon = _icon, label = _label, separator = _separator, bgColor = _bgColor, fontColor = _fontColor;

- (id)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier
{
    self = [super initWithStyle:style reuseIdentifier:reuseIdentifier];
    if (self) {
        
        UIColor* bgColor = self.bgColor? self.bgColor : [UIColor clearColor];
        self.backgroundColor = bgColor;
        
        //点击时不要效果
        self.selectionStyle = UITableViewCellSelectionStyleNone;
        
        //图片
        _icon = [[UIImageView alloc] initWithFrame:CGRectZero];
        _icon.backgroundColor = bgColor;
        [self.contentView addSubview:_icon];
        
        //文本
        UIFont *font = [UIFont systemFontOfSize:17];
        _label = [[UILabel alloc] initWithFrame:CGRectZero];
        _label.textColor = [UIColor whiteColor];
        _label.font = font;
        [self.contentView addSubview:_label];
        
        //自己画分割线
        _separator = [[UIView alloc] initWithFrame:CGRectZero];
        _separator.backgroundColor =  [UIColor grayColor];
        
        [self.contentView addSubview:_separator];
    }
    return self;
}

- (void)setWidth:(CGFloat)width
{
    if (width <= 0) {
        return;
    }
    _width = width;
    self.frame = CGRectMake(0, 0, _width, MENU_CELL_DEFAULT_HEIGHT);
}

- (void)setBgColor:(UIColor *)bgColor
{
    if (!bgColor) {
        return;
    }
    _bgColor = bgColor;
    [self setBackgroundColor:_bgColor];
}

- (void)setFontColor:(UIColor *)fontColor
{
    if (!fontColor) {
        return;
    }
    _fontColor = fontColor;
    _label.textColor = _fontColor;
}

- (void)setNeedSeparetor:(BOOL)needSeparetor
{
    _needSeparetor = needSeparetor;
    [_separator setHidden:!_needSeparetor];
}

- (void)layoutSubviews
{
    UIColor* bgColor = self.bgColor? self.bgColor : [UIColor clearColor];
    self.backgroundColor = bgColor;
    
    CGFloat width = CELL_DEFAULT_WIDTH;
    if (_width > 0) {
        width = _width;
    }
    _backgroundView.frame = CGRectMake(0, 0, width, MENU_CELL_DEFAULT_HEIGHT);
    _icon.frame = ICON_FRAME;
    _label.frame = CGRectMake(2*SPACE+MENU_ICON_DEFAULT_WIDTH, (MENU_CELL_DEFAULT_HEIGHT-LABEl_HEIGHT)/2, width - MENU_ICON_DEFAULT_WIDTH-3*SPACE, LABEl_HEIGHT);
    _separator.frame = CGRectMake(0, MENU_CELL_DEFAULT_HEIGHT - 0.5, width, 0.5);
}
@end
