//
//  menuView.h
//  eimEngine
//
//  Created by CLARKYJ on 17/1/14.
//  Copyright © 2017年 tencent. All rights reserved.
//

#import <UIKit/UIKit.h>
#import "menuViewModel.h"
#import "menuCell.h"

#define MENU_SWITCH 1
#define GlobalClearColor [UIColor clearColor]

/*
 用来标识view出现在屏幕的什么位置，上下左右
 要配合传入的坐标来使用，传入坐标后，指定view
 */
typedef enum : NSUInteger {
    menuView_Left,
    menuView_Right,
    menuView_up,
    menuView_down,
} menuViewAxis;

/*
 用来标识view的箭头出现在什么位置，根据距离坐标点的位置来确定，需要在传入的时候就定义好
 图示如下
                /\     /\     /\
             ---   ----  -----  --
            <                      >
             |                    |
            <                      >
             |                    |
            <                      >
             |                    |
             --    ----  -----  ---
                \/     \/     \/
 */
typedef enum : NSUInteger {
    menuTriangle_Near,
    menuTriangle_Middle,
    menuTriangle_Far,
} menuTriangleDistance;

typedef enum : NSUInteger {
    menuAnimation_Left,
    menuAnimation_LeftTop,
    menuAnimation_LeftBottom,
    menuAnimation_Right,
    menuAnimation_RightTop,
    menuAnimation_RightBttom,
    menuAnimation_Bottom,
    menuAnimation_Top,
} menuAnimationDirection;


@protocol MenuViewDelegate <NSObject>
- (void) onDismiss;
@optional
//菜单项响应函数
- (void)menuCellDidClick: (NSNumber*)cellID;
@end
/////////////////////////////////////////////////

//这个model作为menuView的传入参数，
@interface menuRegInfo : NSObject
@property (nonatomic, assign) menuViewAxis viewMainAxis; //主轴，箭头出现在这个轴上
@property (nonatomic, assign) menuViewAxis viewViceAxis; //副轴，配合主轴用来确定view的象限
@property (nonatomic, assign) menuTriangleDistance viewDistance; //箭头距离
@property (nonatomic, assign) CGPoint  viewBeginPoint;//view的起始点坐标
@property (nonatomic, retain) UIColor* bgColor; //这个view的背景色设置
@property (nonatomic, assign) uint32_t width; //这个tableViewCell的宽度
@property (nonatomic, assign) menuAnimationDirection animationDirection;//动画的方向
@end

@interface menuView : UIView <UIGestureRecognizerDelegate, UITableViewDataSource, UITableViewDelegate>
{
    BOOL _isShow ;
    UIView *_layerView;
    
    NSMutableArray *_dataSource;
    UITableView *_menuTableView;
    __weak id<MenuViewDelegate> _delegate;
    
    menuRegInfo* _regInfo;
}
@property(strong,nonatomic) NSMutableArray *dataSource;
@property(strong,nonatomic) UITableView *menuTableView;
@property(weak,nonatomic) id<MenuViewDelegate> delegate;
@property(nonatomic,retain) menuRegInfo* regInfo;

- (void)slideOut;
- (void)slideIn;
- (BOOL)isShow;
- (void)show;
- (void)dismiss;

//初始化方法
- (id)initWithFrame:(CGRect)frame regInfo:(menuRegInfo*)regInfo;

//添加菜单项, 菜单项响应统一在menuCellDidClick中处理,需实现QQRightMenuViewDelegate.
- (void) addMenuCell:(NSString*)title iconName:(NSString*)iconName cellID:(NSInteger)cellID;

- (void)addMenuCell:(menuViewModel*)cellModel;
@end
